
/*
** CMDTIME.C - Substitute time / date parameters into command lines
** $Id: cmdtime.c,v 1.4 2003/04/20 23:12:39 bob Exp $
*/

/*
** Copyright (C) 2003, R.S. Digital Solutions. All rights reserved.
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License as
** published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public
** License along with this program; if not, write to:
**
**     The Free Software Foundation, Inc.
**     59 Temple Place, Suite 330
**     Boston, MA  02111-1307
**     United States of America
*/

#define TITLE   "CMDTIME"       /*  Program Name    */
#define DATES   "2003"          /*  Copyright Date  */
#define VERSION "1.10"          /*  Version Number  */

/*
** Identify target operating system
*/
#ifdef OS2
#define OPSYSTEM "OS/2"
#endif
#ifdef DOS
#define OPSYSTEM "DOS"
#endif
#ifdef WIN
#define OPSYSTEM "Windows"
#endif
#ifdef LINUX
#define OPSYSTEM "Linux"
#pragma pack(1)
#endif

/*
** Standard includes
*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef LINUX
#include <unistd.h>
#else
#include <mem.h>
#endif

/*
** Program specific includes
*/

/*
** Function prototypes
*/
void ShowError(int);

/*
** Global variables
*/
char *CVS_ID = "$Id: cmdtime.c,v 1.4 2003/04/20 23:12:39 bob Exp $";

/*
** Main program entry
*/
int main(argc, argv)
int argc;
char *argv[];
{
  int i, j, k, n, echoflag, cmdflag;
  time_t secs_now;
  struct tm *tm_now;
  char s_a1[5], s_a2[11], s_b1[5], s_b2[11], s_c1[20], s_c2[22], s_d1[4],
    s_h2[4], s_i2[4], s_j1[5], s_m1[4], s_m2[4], s_p1[4], s_s2[4], s_t2[10],
    s_u2[4], s_w1[3], s_w2[4], s_x1[10], s_x2[12], s_y1[4], s_y2[6], s_z2[10],
    tempstr[1024], junk[30], cmdstring[4096];

  time(&secs_now);
  tm_now = localtime(&secs_now);

  strftime(s_a1, 5, "%a", tm_now);
  strftime(s_a2, 11, "%A", tm_now);
  strftime(s_b1, 5, "%b", tm_now);
  strftime(s_b2, 11, "%B", tm_now);
  strftime(s_c1, 20, "%m-%d-%y %H:%M:%S", tm_now);
  strftime(s_c2, 22, "%Y-%m-%d %H:%M:%S", tm_now);
  strftime(s_d1, 4, "%d", tm_now);
  strftime(s_h2, 4, "%H", tm_now);
  strftime(s_i2, 4, "%I", tm_now);
  strftime(s_j1, 5, "%j", tm_now);
  strftime(s_m1, 4, "%m", tm_now);
  strftime(s_m2, 4, "%M", tm_now);
  strftime(s_p1, 4, "%p", tm_now);
  strftime(s_s2, 4, "%S", tm_now);
  strftime(s_t2, 10, "%H:%M:%S", tm_now);
  strftime(s_u2, 4, "%U", tm_now);
  strftime(s_w1, 3, "%w", tm_now);
  strftime(s_w2, 4, "%W", tm_now);
  strftime(s_x1, 10, "%m-%d-%y", tm_now);
  strftime(s_x2, 12, "%Y-%m-%d", tm_now);
  strftime(s_y1, 4, "%y", tm_now);
  strftime(s_y2, 6, "%Y", tm_now);
  strftime(s_z2, 10, "%Z", tm_now);

  echoflag = 0;
  cmdflag = 0;

  if (argc < 2)
    ShowError(-1);
  for (i = 1; i < argc; i++) {
    if (cmdflag == 0 && argv[i][0] == '-' && argv[i][1] == '-'
        && tolower(argv[i][2]) == 'h') {
      ShowError(0);
    }
    else if (cmdflag == 0 && argv[i][0] == '-' && argv[i][1] == '-'
             && tolower(argv[i][2]) == 'e') {
      echoflag = 1;
    }
    else {
      cmdflag = 1;
      tempstr[0] = '\0';
      j = 0;
      k = 0;
      while (argv[i][j] != '\0') {
        if (argv[i][j] == '@') {
          switch (argv[i][++j]) {
            case '@':
              strcpy(junk, "@");
              break;
            case 'a':
              strcpy(junk, s_a1);
              break;
            case 'A':
              strcpy(junk, s_a2);
              break;
            case 'b':
              strcpy(junk, s_b1);
              break;
            case 'B':
              strcpy(junk, s_b2);
              break;
            case 'c':
              strcpy(junk, s_c1);
              break;
            case 'C':
              strcpy(junk, s_c2);
              break;
            case 'd':
              strcpy(junk, s_d1);
              break;
            case 'H':
              strcpy(junk, s_h2);
              break;
            case 'I':
              strcpy(junk, s_i2);
              break;
            case 'j':
              strcpy(junk, s_j1);
              break;
            case 'm':
              strcpy(junk, s_m1);
              break;
            case 'M':
              strcpy(junk, s_m2);
              break;
            case 'p':
              strcpy(junk, s_p1);
              break;
            case 'q':
              strcpy(junk, "'");
              break;
            case 'Q':
              strcpy(junk, "\"");
              break;
            case 'S':
              strcpy(junk, s_s2);
              break;
            case 'T':
              strcpy(junk, s_t2);
              break;
            case 'U':
              strcpy(junk, s_u2);
              break;
            case 'w':
              strcpy(junk, s_w1);
              break;
            case 'W':
              strcpy(junk, s_w2);
              break;
            case 'x':
              strcpy(junk, s_x1);
              break;
            case 'X':
              strcpy(junk, s_x2);
              break;
            case 'y':
              strcpy(junk, s_y1);
              break;
            case 'Y':
              strcpy(junk, s_y2);
              break;
            case 'Z':
              strcpy(junk, s_z2);
              break;
            default:
              ShowError(254);
          }
          n = 0;
          while (junk[n] != '\0') {
            tempstr[k++] = junk[n++];
          }
	  j++;
        }
        else {
          tempstr[k++] = argv[i][j++];
        }
      }
      tempstr[k] = 0;

      if (cmdstring[0] != '\0')
        strcat(cmdstring, " ");
      if (tempstr[0] == '\0')
        strcat(cmdstring, "\"");
      strcat(cmdstring, tempstr);
      if (tempstr[0] == '\0')
        strcat(cmdstring, "\"");
    }
  }

  if (echoflag == 1)
    printf("Command: %s\n", cmdstring);
  k = system(cmdstring);

  return (k);
}

/*
** Prints error and help screen.
*/
void ShowError(ernum)
int ernum;
{
  printf("\n" TITLE " (" OPSYSTEM " version " VERSION ")\n");
  printf("Copyright " DATES ", Bob Swift.\n\n");
  printf("This program is used to substitute time and ");
  printf("date parameters into command\nline arguments.\n\n");
  printf("Usage:  " TITLE " [--e[cho]] [--h[elp]] command [argument...]\n\n");

  switch (ernum) {

    case 0:
      printf(
        "Within the command and argument strings, the following substitutions will be\n"
        "made, based on the current system time and date:\n"
        "\n"
        "  @a - The abbreviated weekday name\n"
        "  @A - The full weekday name\n"
        "  @b - The abbreviated month name\n"
        "  @B - The full month name\n"
        "  @c - Standard date and time string (mm-dd-yy hh:mm:ss)\n"
        "  @C - Standard date and time string (yyyy-mm-dd hh:mm:ss)\n"
        "  @d - Day-of-month as an integer (01-31)\n"
        "  @H - Hour, in the range of 00-23\n"
        "  @I - Hour, in the range of 01-12\n"
        "  @j - Day-of-year as an integer (001-366)\n"
        "  @m - Month as an integer (01-12)\n"
        "  @M - Minute as an integer (00-59)\n"
        "  @p - Local AM or PM as a string\n"
        "  @q - Single quote character\n"
        "  @Q - Double quote character\n"
        "  @S - Second as an integer (00-59)\n"
        "  @T - Standard time string (hh:mm:ss)\n"
        "  @U - Week-of-year, Sunday being the first day (00-52)\n"
        "  @w - Weekday as an integer (0-6), Sunday being 0\n"
        "  @W - Week-of-year, Monday being the first day (00-52)\n"
        "  @x - Standard date string (mm-dd-yy)\n"
        "  @X - Standard date string (yyyy-mm-dd)\n"
        "  @y - Year as an integer w/o century (00-99)\n"
        "  @Y - Year, including century as an integer\n"
        "  @Z - Time zone (Must have TZ environment variable defined)\n"
        "  @@ - The \"at\" sign\n"
        "\n"
        "Please notice that all options are case-sensitive, and if you wish to enter\n"
        "an \"at\" sign in the command-line, you must specify it twice or else it\n"
        "will be interpreted as one of CMDTIMEx's internal options.\n"
        "\n"
        "The --echo option will echo the resulting command line string to the terminal\n"
        "before executing the command.  The --help option will output a brief help\n"
        "screen regarding the use of the program.  If these options are used, they\n"
        "must appear first, before any other arguments.\n"
        "\n"
        "Any argument that is enclosed in quotes (\"an argument\") will have the\n"
        "quotes stripped by the command processor before passing the argument\n"
        "to CMDTIMEx.  To include quotes in the resulting command passed to the\n"
        "system, use the appropriate @q or @Q substitution options.  Note that\n"
        "null string arguments (\"\") are still passed as null strings enclosed\n"
        "within quotes.\n"
        "\n"
        "Errorlevels returned:    0 - No errors.\n"
        "                       254 - Unknown substitution argument.\n"
        "                        -1 - Missing argument or --help. (Help screen.)\n"
        "\n"
        "When a command is successfully executed, the errorlevel returned from the\n"
        "command is used as the return value from CMDTIME.\n\n"
      );
      break;

    case 254:
      printf("**  Unknown substitution argument [%u]  **\n", ernum);
      break;

    default:
      printf("Use --help for full instructions.\n\n");
  }
  exit(ernum);
}

/*
** -------------------- Revision Information ------------------
**
** $Author: bob $
** $Revision: 1.4 $
** $Date: 2003/04/20 23:12:39 $
**
** --------------------- Revision History ---------------------
**
** $Log: cmdtime.c,v $
** Revision 1.4  2003/04/20 23:12:39  bob
**  - Removed the auto quoting function and replaced with
**    new substitution parameters @q and @Q.
**
** Revision 1.3  2003/04/06 23:16:24  bob
**  - Corrected bug in command line substitution parsing.
**  - Released under GNU General Public License.
**
** Revision 1.2  2003/03/28 16:51:40  bob
**  - Inserted CVS header information.
**  - Changed code state setting to stable.
**
** Revision 1.1.1.1  2003/03/28 16:19:49  bob
**  - Initial check in of project files to CVS
**
** --------------- End of Revision Information ----------------
*/

